@@ -1,6 +1,10 @@
 from __future__ import annotations
 
+from unittest.mock import AsyncMock, MagicMock, patch
+
 import pytest
+from bleak.backends.device import BLEDevice
+from bleak.exc import BleakCharacteristicNotFoundError, BleakError
 from bluetooth_sensor_state_data import BluetoothServiceInfo, DeviceClass, SensorUpdate
 from sensor_state_data import (
     DeviceKey,
@@ -15,7 +19,8 @@ from inkbird_ble.parser import INKBIRDBluetoothDeviceData, Model
 
 
 def test_can_create():
-    INKBIRDBluetoothDeviceData()
+    parser = INKBIRDBluetoothDeviceData()
+    assert parser.name == "Unknown"
 
 
 def test_unsupported():
@@ -33,6 +38,21 @@ def test_unsupported():
     assert parser.device_type is None
 
 
+def test_unsupported_with_manufacturer_data():
+    parser = INKBIRDBluetoothDeviceData()
+    service_info = BluetoothServiceInfo(
+        name="x",
+        manufacturer_data={2044: b"\xc7\x12\x00\xc8=V\x06"},
+        service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        service_data={},
+        source="local",
+    )
+    assert parser.supported(service_info) is False
+    assert parser.device_type is None
+
+
 def test_sps_with_invalid_model_passed():
     parser = INKBIRDBluetoothDeviceData("invalid")
     service_info = BluetoothServiceInfo(
@@ -1101,6 +1121,8 @@ def test_ith_11_b():
         source="local",
     )
     result = parser.update(service_info)
+    assert parser.name == "ITH-11-B EEFF"
+    assert parser.poll_needed(service_info, None) is False
     assert parser.device_type is Model.ITH_11_B
     assert result == SensorUpdate(
         title=None,
@@ -1234,3 +1256,304 @@ def test_ith_11_b():
         binary_entity_values={},
         events={},
     )
+
+
+def test_passive_data_needs_polling() -> None:
+    """Test passive data need polling."""
+    parser = INKBIRDBluetoothDeviceData(Model.IBS_TH)
+    assert parser.device_type == Model.IBS_TH
+    service_info = BluetoothServiceInfo(
+        name="N0BYD",
+        manufacturer_data={},
+        service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        service_data={},
+        source="local",
+    )
+    result = parser.update(service_info)
+    assert parser.poll_needed(service_info, None) is True
+    assert parser.name == "IBS-TH EEFF"
+    assert result == SensorUpdate(
+        title=None,
+        devices={
+            None: SensorDeviceInfo(
+                name="IBS-TH EEFF",
+                model="IBS-TH",
+                manufacturer="INKBIRD",
+                sw_version=None,
+                hw_version=None,
+            )
+        },
+        entity_descriptions={
+            DeviceKey(key="signal_strength", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                device_class=SensorDeviceClass.SIGNAL_STRENGTH,
+                native_unit_of_measurement=Units.SIGNAL_STRENGTH_DECIBELS_MILLIWATT,
+            ),
+        },
+        entity_values={
+            DeviceKey(key="signal_strength", device_id=None): SensorValue(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                name="Signal Strength",
+                native_value=-60,
+            ),
+        },
+        binary_entity_descriptions={},
+        binary_entity_values={},
+        events={},
+    )
+
+
+@pytest.mark.asyncio
+async def test_passive_polling_ibs_th() -> None:
+    """Test polling with passing data."""
+    parser = INKBIRDBluetoothDeviceData(Model.IBS_TH)
+    assert parser.device_type == Model.IBS_TH
+    service_info = BluetoothServiceInfo(
+        name="N0BYD",
+        manufacturer_data={},
+        service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        service_data={},
+        source="local",
+    )
+    parser.update(service_info)
+    assert parser.poll_needed(service_info, None) is True
+    read_gatt_char_mock = AsyncMock(return_value=b"\x09\x09\x00\x04\xe37\x08")
+    disconnect_mock = AsyncMock()
+    mock_client = MagicMock(
+        read_gatt_char=read_gatt_char_mock, disconnect=disconnect_mock
+    )
+    with patch("inkbird_ble.parser.establish_connection", return_value=mock_client):
+        update = await parser.async_poll(
+            BLEDevice(
+                address="aa:bb:cc:dd:ee:ff",
+                name="N0BYD",
+                details={},
+                rssi=-60,
+            )
+        )
+    assert update == SensorUpdate(
+        title=None,
+        devices={
+            None: SensorDeviceInfo(
+                name="IBS-TH EEFF",
+                model="IBS-TH",
+                manufacturer="INKBIRD",
+                sw_version=None,
+                hw_version=None,
+            )
+        },
+        entity_descriptions={
+            DeviceKey(key="signal_strength", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                device_class=SensorDeviceClass.SIGNAL_STRENGTH,
+                native_unit_of_measurement=Units.SIGNAL_STRENGTH_DECIBELS_MILLIWATT,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                device_class=SensorDeviceClass.TEMPERATURE,
+                native_unit_of_measurement=Units.TEMP_CELSIUS,
+            ),
+            DeviceKey(key="humidity", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                device_class=SensorDeviceClass.HUMIDITY,
+                native_unit_of_measurement=Units.PERCENTAGE,
+            ),
+        },
+        entity_values={
+            DeviceKey(key="signal_strength", device_id=None): SensorValue(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                name="Signal Strength",
+                native_value=-60,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorValue(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                name="Temperature",
+                native_value=23.13,
+            ),
+            DeviceKey(key="humidity", device_id=None): SensorValue(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                name="Humidity",
+                native_value=10.24,
+            ),
+        },
+        binary_entity_descriptions={},
+        binary_entity_values={},
+        events={},
+    )
+
+
+@pytest.mark.asyncio
+async def test_passive_polling_ith_11_b() -> None:
+    """Test polling with passing data."""
+    parser = INKBIRDBluetoothDeviceData(Model.ITH_11_B)
+    assert parser.device_type == Model.ITH_11_B
+    service_info = BluetoothServiceInfo(
+        name="N0BYD",
+        manufacturer_data={},
+        service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        service_data={},
+        source="local",
+    )
+    parser.update(service_info)
+    assert parser.poll_needed(service_info, None) is True
+    read_gatt_char_mock = AsyncMock(return_value=b"rtdth\xd8\x00\xef\x01a\x00\x90\x04")
+    disconnect_mock = AsyncMock()
+    mock_client = MagicMock(
+        read_gatt_char=read_gatt_char_mock, disconnect=disconnect_mock
+    )
+    with patch("inkbird_ble.parser.establish_connection", return_value=mock_client):
+        update = await parser.async_poll(
+            BLEDevice(
+                address="aa:bb:cc:dd:ee:ff",
+                name="N0BYD",
+                details={},
+                rssi=-60,
+            )
+        )
+    assert update == SensorUpdate(
+        title=None,
+        devices={
+            None: SensorDeviceInfo(
+                name="ITH-11-B EEFF",
+                model="ITH-11-B",
+                manufacturer="INKBIRD",
+                sw_version=None,
+                hw_version=None,
+            )
+        },
+        entity_descriptions={
+            DeviceKey(key="signal_strength", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                device_class=SensorDeviceClass.SIGNAL_STRENGTH,
+                native_unit_of_measurement=Units.SIGNAL_STRENGTH_DECIBELS_MILLIWATT,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                device_class=SensorDeviceClass.TEMPERATURE,
+                native_unit_of_measurement=Units.TEMP_CELSIUS,
+            ),
+            DeviceKey(key="humidity", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                device_class=SensorDeviceClass.HUMIDITY,
+                native_unit_of_measurement=Units.PERCENTAGE,
+            ),
+            DeviceKey(key="battery", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="battery", device_id=None),
+                device_class=SensorDeviceClass.BATTERY,
+                native_unit_of_measurement=Units.PERCENTAGE,
+            ),
+        },
+        entity_values={
+            DeviceKey(key="signal_strength", device_id=None): SensorValue(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                name="Signal Strength",
+                native_value=-60,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorValue(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                name="Temperature",
+                native_value=21.6,
+            ),
+            DeviceKey(key="humidity", device_id=None): SensorValue(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                name="Humidity",
+                native_value=49.5,
+            ),
+            DeviceKey(key="battery", device_id=None): SensorValue(
+                device_key=DeviceKey(key="battery", device_id=None),
+                name="Battery",
+                native_value=97,
+            ),
+        },
+        binary_entity_descriptions={},
+        binary_entity_values={},
+        events={},
+    )
+
+
+@pytest.mark.asyncio
+async def test_passive_polling_fails_missing_char() -> None:
+    """Test polling with passing data."""
+    parser = INKBIRDBluetoothDeviceData(Model.ITH_11_B)
+    assert parser.device_type == Model.ITH_11_B
+    service_info = BluetoothServiceInfo(
+        name="N0BYD",
+        manufacturer_data={},
+        service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        service_data={},
+        source="local",
+    )
+    parser.update(service_info)
+    assert parser.poll_needed(service_info, None) is True
+    read_gatt_char_mock = AsyncMock(side_effect=BleakCharacteristicNotFoundError(1))
+    disconnect_mock = AsyncMock()
+    clear_cache_mock = AsyncMock()
+    mock_client = MagicMock(
+        read_gatt_char=read_gatt_char_mock,
+        disconnect=disconnect_mock,
+        clear_cache=clear_cache_mock,
+    )
+
+    with (
+        pytest.raises(BleakCharacteristicNotFoundError),
+        patch("inkbird_ble.parser.establish_connection", return_value=mock_client),
+    ):
+        await parser.async_poll(
+            BLEDevice(
+                address="aa:bb:cc:dd:ee:ff",
+                name="N0BYD",
+                details={},
+                rssi=-60,
+            )
+        )
+
+    clear_cache_mock.assert_awaited_once()
+
+
+@pytest.mark.asyncio
+async def test_passive_polling_fails_generic_bleak_error() -> None:
+    """Test polling with passing data."""
+    parser = INKBIRDBluetoothDeviceData(Model.ITH_11_B)
+    assert parser.device_type == Model.ITH_11_B
+    service_info = BluetoothServiceInfo(
+        name="N0BYD",
+        manufacturer_data={},
+        service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        service_data={},
+        source="local",
+    )
+    parser.update(service_info)
+    assert parser.poll_needed(service_info, None) is True
+    read_gatt_char_mock = AsyncMock(side_effect=BleakError)
+    disconnect_mock = AsyncMock()
+    clear_cache_mock = AsyncMock()
+    mock_client = MagicMock(
+        read_gatt_char=read_gatt_char_mock,
+        disconnect=disconnect_mock,
+        clear_cache=clear_cache_mock,
+    )
+
+    with (
+        pytest.raises(BleakError),
+        patch("inkbird_ble.parser.establish_connection", return_value=mock_client),
+    ):
+        await parser.async_poll(
+            BLEDevice(
+                address="aa:bb:cc:dd:ee:ff",
+                name="N0BYD",
+                details={},
+                rssi=-60,
+            )
+        )
+
+    clear_cache_mock.assert_not_awaited()
